#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of ITango (http://pypi.python.org/pypi/itango)
#
# Copyright 2006-2012 CELLS / ALBA Synchrotron, Bellaterra, Spain
# Copyright 2013-2014 European Synchrotron Radiation Facility, Grenoble, France
#
# Distributed under the terms of the GNU Lesser General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

"""functions common (hopefully) to all ipython versions"""

__all__ = ["get_python_version",
           "get_ipython_version",
           "get_pytango_version"]

import sys
from distutils.version import StrictVersion


# Python utilities

def get_python_version():
    return StrictVersion('.'.join(map(str, sys.version_info[:3])))


# IPython utilities

def get_ipython_version():
    """Returns the current IPython version"""
    import IPython
    v = None
    if hasattr(IPython, "version_info"):
        v = '.'.join(map(str, IPython.version_info[:3]))
    else:
        try:
            try:
                v = IPython.Release.version
            except:
                try:
                    v = IPython.release.version
                except:
                    pass
        except:
            pass
    return StrictVersion(v)


# PyTango utilities

def get_pytango_version():
    try:
        import tango
    except:
        return
    try:
        v = tango.Release.version
    except:
        v = '0.0.0'
    return StrictVersion(v)



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of ITango (http://pypi.python.org/pypi/itango)
#
# Copyright 2006-2012 CELLS / ALBA Synchrotron, Bellaterra, Spain
# Copyright 2013-2014 European Synchrotron Radiation Facility, Grenoble, France
#
# Distributed under the terms of the GNU Lesser General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from __future__ import with_statement

import os
import sys

import IPython
from IPython.core.profiledir import ProfileDirError, ProfileDir
from IPython.utils.io import ask_yes_no

try:
    from IPython.paths import get_ipython_dir, ensure_dir_exists
except ImportError:
    try:
        from IPython.utils.path import get_ipython_dir, ensure_dir_exists
    except ImportError:
        from IPython.genutils import get_ipython_dir, ensure_dir_exists

import tango

__PROFILE = """\
#!/usr/bin/env ipython
\"\"\"An automaticaly generated IPython profile designed
to provide a user friendly interface to Tango.

Created with PyTango {pytangover} for IPython {ipyver}
\"\"\"

# Protected block
{protected_block}

# Put any additional environment here

"""

__PROTECTED_BLOCK = """\
import itango
config = get_config()
itango.load_config(config)
"""

_CONFIG_FILE_NAME = 'ipython_config.py'


def install_kernel(verbose=True):
    if verbose:
        print('Installing ITango kernelspec for jupyter...')
    # See if ipykernel is available
    try:
        from ipykernel.kernelspec import install as kernelinstall
    except ImportError:
        if verbose:
            print('The ipykernel module is not available; '
                  'ITango kernelspec will not be installed')
        return False
    # Try to install the kernel spec
    try:
        path = kernelinstall(kernel_name='tango', profile='tango',
                             display_name='ITango', user=True)
    except Exception as exc:
        if verbose:
            print('Error while installing ITango kernelspec: {!r}'.format(exc))
        return False
    # Installation OK
    if verbose:
        print('Installed ITango kernelspec in {}'.format(path))
    return True


def is_installed(ipydir=None, profile='tango'):
    ipython_dir = ipydir or get_ipython_dir()
    try:
        p_dir = ProfileDir.find_profile_dir_by_name(ipython_dir, profile)
    except ProfileDirError:
        return False
    abs_config_file_name = os.path.join(p_dir.location, _CONFIG_FILE_NAME)
    if not os.path.isfile(abs_config_file_name):
        return False
    with open(abs_config_file_name) as f:
        return __PROTECTED_BLOCK in f.read()


def install(ipydir=None, verbose=True, profile='tango'):
    if verbose:
        def out(msg):
            sys.stdout.write(msg)
            sys.stdout.flush()
    else:
        out = lambda x: None

    ipython_dir = ipydir or get_ipython_dir()
    ensure_dir_exists(ipython_dir)
    try:
        p_dir = ProfileDir.find_profile_dir_by_name(ipython_dir, profile)
    except ProfileDirError:
        p_dir = ProfileDir.create_profile_dir_by_name(ipython_dir, profile)
    abs_config_file_name = os.path.join(p_dir.location, _CONFIG_FILE_NAME)
    create_config = True
    if os.path.isfile(abs_config_file_name):
        msg = "Tango configuration file {0} already exists.\n"
        msg += "Do you wish to replace it (y/n)?"
        msg = msg.format(abs_config_file_name)
        create_config = ask_yes_no(msg, default='y')

    if not create_config:
        return

    out("Installing tango extension to ipython... ")

    profile = __PROFILE.format(pytangover=tango.Release.version,
                               ipyver=IPython.release.version,
                               protected_block=__PROTECTED_BLOCK)
    with open(abs_config_file_name, "w") as f:
        f.write(profile)
        f.close()
    out("[DONE]\n\n")
    out("""\
To start ipython with tango interface simply type on the command line:
%% ipython --profile=tango

For more information goto:
http://pytango.readthedocs.io

Have fun with ITango!
The PyTango team
""")


def main():
    d = None
    if len(sys.argv) > 1:
        d = sys.argv[1]
    install(d)
    install_kernel()


if __name__ == "__main__":
    main()



"""Sphinx configuration for itango documentation."""

import sys
import os


# Extensions

sys.path.append(os.path.abspath('sphinxext'))
extensions = ['ipython_console_highlighting',
              'tango_console_highlighting']

# Configuration

master_doc = 'itango'
rst_epilog = """\
.. _Tango: http://www.tango-controls.org/
.. _IPython: http://ipython.org/
.. _ITango: https://pypi.python.org/pypi/itango/
"""

# Data

project = u'itango'
copyright = u'2016, Tango Controls'



# -----------------------------------------------------------------------------
# This file is part of ITango (http://pypi.python.org/pypi/itango)
#
# Copyright 2006-2012 CELLS / ALBA Synchrotron, Bellaterra, Spain
# Copyright 2013-2014 European Synchrotron Radiation Facility, Grenoble, France
#
# Distributed under the terms of the GNU Lesser General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

__all__ = ["install",
           "load_ipython_extension", "unload_ipython_extension",
           "init_ipython", "load_config", "run", "run_qt",
           "get_python_version", "get_ipython_version",
           "get_pytango_version"]

from .common import get_python_version, get_ipython_version
from .common import get_pytango_version

from .install import install
from .itango import load_ipython_extension, unload_ipython_extension
from .itango import init_ipython, load_config, run, run_qt



"""Setup file for itango."""

import platform
from setuptools import setup, find_packages


def get_entry_points():
    major = int(platform.python_version_tuple()[0])
    name = 'itango3' if major == 3 else 'itango'
    return {
        "console_scripts": ["{0} = itango:run".format(name)],
        "gui_scripts": ["{0}-qt = itango:run_qt".format(name)]}


CLASSIFIERS = """\
Framework :: IPython
Intended Audience :: Developers
Intended Audience :: Science/Research
License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)
Programming Language :: Python
Programming Language :: Python :: 2
Programming Language :: Python :: 3
Topic :: System :: Shells
""".splitlines()


setup(
    name='itango',
    version='0.1.7',

    packages=find_packages(),
    package_data={'itango': [
        'resource/*.png', 'resource/*.svg']},
    entry_points=get_entry_points(),
    install_requires=[
        'IPython>=1.0',
        'pytango>=9.2.0'],

    license='LGPL',
    classifiers=CLASSIFIERS,
    author='Tiago Coutinho',
    author_email="coutinho@esrf.fr",
    description='An interactive Tango client',
    long_description=open('README.rst').read(),
    url='https://github.com/tango-cs/itango',
    download_url='http://pypi.python.org/pypi/itango',
    platforms=['Linux', 'Windows XP/Vista/7/8'],
    keywords=['PyTango', 'IPython'],
    )



# -----------------------------------------------------------------------------
# This file is part of ITango (http://pypi.python.org/pypi/itango)
#
# Copyright 2006-2012 CELLS / ALBA Synchrotron, Bellaterra, Spain
# Copyright 2013-2014 European Synchrotron Radiation Facility, Grenoble, France
#
# Distributed under the terms of the GNU Lesser General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from __future__ import print_function

import re
import io

LENGTHS = 4, 30, 18, 20, 12, 16
COLUMNS = 'ID', 'Device', 'Attribute', 'Value', 'Quality', 'Time'

TEMPLATE = ' '.join('{%d:%d}' % x for x in enumerate(LENGTHS))
TITLE = TEMPLATE.format(*COLUMNS) + '\n' + ' '.join('-' * l for l in LENGTHS)


class EventLogger(object):

    def __init__(self, capacity=100000, pager=None):
        self._capacity = capacity
        self._pager = pager
        self._records = []

    def push_event(self, evt):
        attr_name = evt.attr_name
        dev, sep, attr = attr_name.rpartition('/')
        if dev.startswith("tango://"):
            dev = dev[8:]
        if dev.count(":"):
            # if it has tango host
            host, sep, dev = dev.partition('/')
        else:
            host = "-----"
        evt.host = host
        evt.dev_name = dev
        evt.s_attr_name = attr
        self._records.append(evt)
        over = len(self._records) - self._capacity
        if over > 0:
            self._records = self._records[over:]

    def model(self):
        return self

    def getEvents(self):
        return self._records

    def show(self, dexpr=None, aexpr=None):
        if dexpr is not None:
            dexpr = re.compile(dexpr, re.IGNORECASE)
        if aexpr is not None:
            aexpr = re.compile(aexpr, re.IGNORECASE)

        class StringIO(io.StringIO):
            def write(self, value):
                if isinstance(value, bytes):
                    value = value.decode()
                io.StringIO.write(self, value)

        s = StringIO()

        print(TITLE, file=s)

        for i, r in enumerate(self._records):
            if dexpr is not None and not dexpr.match(r.dev_name):
                continue
            if aexpr is not None and not aexpr.match(r.s_attr_name):
                continue
            if r.err:
                v = r.errors[0].reason
                q = 'ERROR'
                ts = r.reception_date.strftime("%H:%M:%S.%f")
            else:
                v = str(r.attr_value.value)
                q = str(r.attr_value.quality)
                ts = r.attr_value.time.strftime("%H:%M:%S.%f")
            args = i, r.dev_name, r.s_attr_name, v, q, ts
            print(TEMPLATE.format(*args), file=s)
        s.seek(0)
        if self._pager is None:
            print(s.read())
        else:
            self._pager(s.read())
